{--------------------------------------------------------}

{
{    Tratamento Dicvox, usa as bases de dados criadas pelo Glauco Frius
{
{    Autor: Neno Henrique da Cunha Albernaz
{
{    Em 27/06/2020
{
{--------------------------------------------------------}

Unit edDicvox;

interface

uses
    DVWin, dvCrt, windows, sysUtils, classes,
    dvexec, dvForm,
    edDicion, edVars, edUtil, edMensag;

procedure trataDicvox;

implementation

{--------------------------------------------------------}

procedure abrirDicvox (progDicvox, palavraBuscar, dicEscolhido: string);
begin
    if executaProg ('"' + progDicvox + '" /' + dicEscolhido, '', palavraBuscar) >= 32 then
        esperaProgVoltar;
    while sintFalando do waitMessage;
    limpaBufTec;
    fala ('EDOK'); {'OK'}
end;

{--------------------------------------------------------}

function menuTraduzir: string;
const
    tabTraduz: string =      // Cdigos dos dicionrios no Dicvox:
                                   'I' + // Ingls - Portugus - Ingls
                                   '1' + //Portugus - Ingls - Portugus
                                   'D' + // Espanhol - Portugus - Espanhol
                                   '4' + // Portugus - Espanhol - Portugus
                                   'T' + // Italiano - Portugus - Italiano
                                   '7' + // Portugus - Italiano - Portugus
                                   'F' + // Francs - Portugus - Francs
                                   'V' + // Alemo - Portugus
                                   'X' + // Esperanto - Portugus
                                   '5' + // Latim - Ingls - Latim
                                   '6' + // Ingls - Latim - Ingls
                                   'S';  // Ingls - Espanhol'

var
    nSel, i, numTraduz: integer;
begin
    sintclek;
    numTraduz := length(tabTraduz);
    popupMenuCria(40, 9, 30, numTraduz, RED);
        for i := 1 to numTraduz do
            popupMenuAdiciona ('EDAJDT' + intToStr(i), txtmsg ('EDAJDT' + intToStr(i)));
    nSel := popupMenuSeleciona;

    if (nSel > 0) and (nSel <= numTraduz) then
        result := tabTraduz[nSel]
    else
        result := '';
end;

{--------------------------------------------------------}

function menuOutros: string;
const
    tabOutros: string =      // Cdigos dos dicionrios no Dicvox:
                                   'M' + // Significado e origem dos nomes
                                   'R' + // Origem da Palavra
                                   '0' + // Origem da Palavra 2
                                   '8' + // Etimolgico - Origem das Palavras
                                   'A' + // Jurdico de Latim
                                   'B' + // Bblia de Almeida
                                   'K' + // Eletrnica em espanhol
                                   '2' + // Grego em portugus com Concordncia em grego nas informaes
                                   'G' + // Grego com Concordncia em grego nas informaes
                                   '3' + // Hebraico em portugus com Concordncia em hebraico nas informaes
                                   'H' + // Hebraico com Concordncia em hebraico nas informaes
                                   '9';  // Bblico da Torre de Vigia

var
    nSel, i, numOutros: integer;
begin
    sintclek;
    numOutros := length(tabOutros);
    popupMenuCria(40, 9, 30, numOutros, RED);
        for i := 1 to numOutros do
            popupMenuAdiciona ('EDAJDO' + intToStr(i), txtmsg ('EDAJDO' + intToStr(i)));
    nSel := popupMenuSeleciona;

    if (nSel > 0) and (nSel <= numOutros) then
        result := tabOutros[nSel]
    else
        result := '';
end;

{--------------------------------------------------------}

procedure trataDicvox;
const
    tabPrincipal: string =      // Cdigos dos dicionrios no Dicvox:
                                   'P' + // Portugus
                                   'Y' + // Vocabulrio Ortogrfico da Lngua Portuguesa (VOLP)
                                   'W' + // Webster
                                   'O' + // Oxford
                                   'E' + // Espanhol - Espanhol
                                   'N' + // Informtica
                                   'J' + // Jurdico Brasileiro
                                   'U' + // Psicologia
                                   'L' + // Filosofia
                                   'C' + // Sociologia
                                   'Z' + // Qumica
                                   'Q';  // Significados

var
    s, dicSel, palavra, progDicvox: string;
    i, x, nSel, numPrin: integer;
begin
    progDicvox := sintAmbiente ('EDIVOX', 'PROGDICVOX', 'C:\Dicvox\DicvoxExtra.exe');
    if not FileExists(progDicvox) then
        begin
            fala ('EDPRONEN'); {'Programa no encontrado'}
            sintetiza (progDicvox);
            exit;
        end;

     if (posy <= 0) then exit;
     s := texto[posy];
    x := posx;
    while (x > 1) and (s[x-1] = ' ') do
        x := x - 1;
    while (x <= length(s)) and (s[x] in LETRAS_DE_PALAVRA) do
        x := x + 1;
     palavra := descobrePalavraAntes (x);
    if palavra = '' then
        begin
            sintBip;
            exit;
        end;

    fala ('EDDICION');    {'Dicionrios - use as setas para selecionar.'}
    sintclek;
    numPrin := length(tabPrincipal);
    popupMenuCria(40, 9, 30, (numPrin + 2), RED);
        for i := 1 to numPrin do
            popupMenuAdiciona ('EDAJDP' + intToStr(i), txtmsg ('EDAJDP' + intToStr(i)));
        popupMenuAdiciona ('', 'Traduo');
        popupMenuAdiciona ('', 'Outros');
    nSel := popupMenuSeleciona;

    if (nSel > 0) and (nSel <= numPrin) then
        dicSel := tabPrincipal[nSel]
    else
    if nSel = (numPrin + 1) then
        dicSel := menuTraduzir
    else
    if nSel = (numPrin + 2) then
        dicSel := menuOutros
    else
        dicSel := '';

    if dicSel = '' then
        fala ('EDDESIST')    {'Desistiu'}
    else
        abrirDicvox (progDicvox, palavra, dicSel);
end;

{--------------------------------------------------------}

begin
end.
